/* Account.cc - source file for the mailfilter program
 * Copyright (c) 2000  Andreas Bauer <baueran@in.tum.de>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307,
 * USA.
 */


#ifndef ACCOUNT_HH
#define ACCOUNT_HH

#include <string>
#include <vector>
#include "Preferences.hh"
#include "Feedback.hh"

#define MAX_BYTES 512

namespace acc {

  typedef struct {
    string* descr;
    string* content;
  } Line;

  typedef struct {
    int number;
    int size;
    vector<Line> lines;

    // We store those separately so it's easier to log the information
    string* sender; 
    string* subject;
    string* date;
  } Header;
  
  class Account {
  private:
    // Variables
    vector<Header> headers;
    string* server;
    string* user;
    string* pass;
    int port;
    int mySocket;
    pref::Preferences* prefs;
    fb::Feedback *report;

    // Methods
    int login();
    int connectSocket();
    int disconnectServer();
    int deleteMessage(int);
    string sendReceive(string);
    string sendReceiveB(string);
    int storeMessageHeader(string, int, int);
    string getWord(string, int);
    string cleanString(string);

  public:
    Account(string, string, string, int, pref::Preferences*);
    ~Account();
    int check();
  };

  // Exceptions
  class CommunicationException { };
}  

#endif
