/* Preferences.cc - source file for the mailfilter program
 * Copyright (c) 2000  Andreas Bauer <baueran@in.tum.de>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307,
 * USA.
 */

#include <string>
#include <list>
#include <fstream>
#include <stdlib.h>
#include "Preferences.hh"
#include "mailfilter.hh"

namespace pref {

  Preferences::Preferences(string file) {
    string curLine;
    serverInfo popServer;
    maxsize = 0;
    mode = VERBOSE;
    ifstream prefsFile(file.c_str());
    
    if (prefsFile) {
      while (getline(prefsFile, curLine)) {
	if ( (curLine.find("SERVER") == 0) && (curLine.find("#") == -1) )
	  popServer.name = *(getPrefValues(curLine)).begin();
	else if ( (curLine.find("USER") == 0) && (curLine.find("#") == -1) )
	  popServer.user = *(getPrefValues(curLine)).begin();
	else if ( (curLine.find("PASS") == 0) && (curLine.find("#") == -1) )
	  popServer.pass = *(getPrefValues(curLine)).begin();
	else if ( (curLine.find("PORT") == 0) && (curLine.find("#") == -1) ) {
	  popServer.port = atoi( (*(getPrefValues(curLine)).begin()).c_str() );
	  servers.push_back(popServer);    // always push here, cause after this either another server gets configured, or we're done
	}
	else if ( (curLine.find("MAXSIZE") == 0) && (curLine.find("#") == -1) )
	  maxsize = atoi( (*(getPrefValues(curLine)).begin()).c_str() );
	else if ( (curLine.find("LOGFILE") == 0) && (curLine.find("#") == -1) )
	  logfile = (*(getPrefValues(curLine)).begin());
	else if ( (curLine.find("MODE") == 0) && (curLine.find("#") == -1) ) {
	  if (*(getPrefValues(curLine)).begin() == "silent")
	    mode = SILENT;
	  else
	    mode = VERBOSE;
	}
	else if ( curLine.find("DENY") == 0 )
	  filters.push_back(getFilter(curLine));
      }
    }
    else
      throw IOException();
    
    prefsFile.close();
  }
  
  
  Preferences::~Preferences() {
  }
  
  
  vector<serverInfo> Preferences::getServers() {
    return servers;
  }
  
  
  // Gets the values from an entry inside the preferences file
  // e.g. TEST=value1, value2
  // would result in returning 'value1' and 'value2' (I've stolen this from my own application Archiebald)
  list<string> Preferences::getPrefValues(string prefLine) {
    list<string> parsedValues;
    int startSearch = prefLine.find('=') + 1;
    int newPos = 0;
    
    if (startSearch != -1) {
      for (int i = startSearch; i <= prefLine.length(); i++) {
	newPos = prefLine.find(',', i);
	
	if (newPos != -1) {
	  parsedValues.push_back( prefLine.substr( i, newPos - i ) );
	  i = newPos + 1;
	}
	else {
	  parsedValues.push_back( prefLine.substr( i, prefLine.length() ) );
	  break;
	}
      }
      
      return parsedValues;
    }
    else throw MalformedPrefsFile();
  }


  int Preferences::getMaxsize() {
    return maxsize;
  }


  // Extract and compile filter information from preferences file
  string Preferences::getFilter(string curLine) {
    int start = curLine.find('=') + 1;

    if (start != -1) {
      return(curLine.substr(start));
    }
    else
      throw MalformedPrefsFile();
  }


  string Preferences::getLogfile() {
    return logfile;
  }


  void Preferences::setLogfile(string newLogfile) {
    logfile = newLogfile;
  }

  int Preferences::getMode() {
    return mode;
  }


  void Preferences::setMode(int newMode) {
    mode = newMode;
  }


  // Return vector with all compiled filters
  vector<string>* Preferences::getFilters() {
    return &filters;
  }

}
